--[[
Name: CooldownButtons
Project Revision: 7d8ae698db10950ea1dd2f01ef6d777d0cb41e2d
File Revision: e1cab2dfffd7d0a613abd6948154c843ca4ddbc5 
Author(s): Netrox
Website: http://www.wowace.com/projects/cooldownbuttons/
Git Repo: git://git.wowace.com/wow/cooldownbuttons/mainline.git
License: All rights reserved.
]]

local CDB = CDB
local spells = CreateFrame('frame')
CDB.spells = spells

spells:SetScript("OnEvent", function(self, event, ...) if self[event] then return self[event](self, event, ...) end end)

--local L = CooldownButtons.L

------
local DF
local newList, newDict, del, deepDel, deepCopy = CDB.GetRecyclingFunctions()
local string_find = string.find
------

function spells:Init()
    DF = LibStub("LibDeformat-3.0-FORK")
    
    self.tooltip = CreateFrame("GameTooltip", "CDBTooltip", UIParent, "GameTooltipTemplate")
    self.tooltip:SetOwner(UIParent, "ANCHOR_NONE")

    self.db = CDB.db
    self.spellTable = newList()
    self.treeTable  = newList()

    self:RegisterEvent("SPELL_UPDATE_COOLDOWN")
    self:RegisterEvent("PET_BAR_UPDATE_COOLDOWN")
    self:RegisterEvent("PLAYER_TALENT_UPDATE")
    self:RegisterEvent("SPELLS_CHANGED")


    self:SPELLS_CHANGED() -- Init Spell Table
    self:SPELL_UPDATE_COOLDOWN()
    self:PET_BAR_UPDATE_COOLDOWN()
end

function spells:SPELL_UPDATE_COOLDOWN()
    local hasNewCooldown = false
    local treeTable   = self.treeTable
    local spellTable  = self.spellTable
    local spellsToAdd = newList()
    local spellTreeTable = newList(newList(),newList(),newList(),newList())
    for _, spellData in pairs(self.spellTable) do
        local spellID = spellData.spellID
        local spellName  = spellData.spellName
        local start, duration, enable = GetSpellCooldown(spellID)
        if enable == 1 and start > 0 and duration >= 3 then
            spellTreeTable[spellData.spellTree][start*duration] = 1 + (spellTreeTable[spellData.spellTree][start*duration] or 0)
            spellsToAdd[spellID] = newList(start, duration)
        end
    end
    for treeIndex = 1, 2 do -- General Tab and current Spec
        for time in pairs(spellTreeTable[treeIndex]) do
            if spellTreeTable[treeIndex][time] > 3 then
                local spellID = nil
                for id, data in pairs(spellsToAdd) do
                    if treeIndex == spellTable[id].spellTree then
                        if data[1] * data[2] == time then
                            spellID = id
                            spellsToAdd[id] = del(spellsToAdd[id])
                        end
                    end
                end
                local treeName    = treeTable[treeIndex].treeName
                local treeTexture = treeTable[treeIndex].treeTexture

                if CDB:AddCooldown("Spell", treeName, spellID, treeTexture) then
                    hasNewCooldown = true
                end
            end
        end
    end
    for spellID in pairs(spellsToAdd) do
        local spellName   = self.spellTable[spellID].spellName
        local spellTexture = self.spellTable[spellID].spellTexture
        
        if CDB:AddCooldown("Spell", spellName, spellID, spellTexture) then
            hasNewCooldown = true
        end
    end
    if hasNewCooldown == true then
        CDB:SortCooldowns()
    end

    --cleanup
    spellsToAdd = deepDel(spellsToAdd)
    spellTreeTable = deepDel(spellTreeTable)
end

function spells:PET_BAR_UPDATE_COOLDOWN()
    local hasNewCooldown = false
    if not UnitExists("pet") --[[ oder config sagt keine pets !]] then return end
    for spellIndex = 0, 9 do
        local spellName = GetPetActionInfo(spellIndex)
        if spellName ~= nil then
            local start, duration, enable = GetPetActionCooldown(spellIndex)
            if enable == 1 and start > 0 and duration >= 3 then
                local texture = select(2, GetPetActionInfo(spellIndex))
                local spellID = select(7, GetPetActionInfo(spellIndex))
                if CDB:AddCooldown("PetAction", spellName, spellID, texture) then
                    hasNewCooldown = true
                end
            end
        end
    end
    if hasNewCooldown == true then
        CDB:SortCooldowns()
    end
end

function spells:PLAYER_TALENT_UPDATE()
    self:SPELLS_CHANGED()
end

function spells:SPELLS_CHANGED()
    local treeTable   = self.treeTable
    for spellTree = 1, 2 do -- General Tab and current Spec
        local treeName, treeTexture, offset, num = GetSpellTabInfo(spellTree)
        treeTable[spellTree] = newDict(
            "treeName"    , treeName,
            "treeTexture" , treeTexture
        )
        for j = 1, num do
            local spellIndex = offset + j
            local spellName, spellID, texture = self:GetSpellInfo(spellIndex, BOOKTYPE_SPELL)
            if spellName ~= nil then
            	self:insertSpell(spellName, spellID, texture, spellTree)
            else
                local _, _, flyoutSlots, flyoutIsKnown = GetFlyoutInfo(spellID);
                if flyoutIsKnown then
                    for slot = 1, flyoutSlots do
                        local spellID1, spellID2, isKnown = GetFlyoutSlotInfo(spellID, slot)
                        if isKnown then
                            local spellName1, _, texture1 = GetSpellInfo(spellID1)
                            if spellName1 ~= nil then
                                self:insertSpell(spellName1, spellID1, texture1, spellTree)
                            end
                            if spellID1 ~= spellID2 then
                                local spellName2, _, texture2 = GetSpellInfo(spellID2)
                                if spellName2 ~= nil then
                                    self:insertSpell(spellName2, spellID2, texture2, spellTree)
                                end
                            end
                        end
                    end
                end
            end
        end
    end
end

function spells:insertSpell(spellName, spellID, texture, spellTree)
	if not self.spellTable[spellID] then
		self.spellTable[spellID] = newDict(
			"spellName"    , spellName,
			"spellID"      , spellID,
			"spellKnownCD" , self:GetKnownCooldown(spellID),
			"spellTexture" , texture,
			"spellTree"    , spellTree
		)
		local _, class = UnitClass("player")
		if class == "DEATHKNIGHT" and self.spellTable[spellID].spellKnownCD == nil then
			self.spellTable[spellID] = del(self.spellTable[spellID])
        end
    else
        -- try to refresh known cooldown duration
        self.spellTable[spellID].spellKnownCD = self:GetKnownCooldown(spellID)
	end
end

function spells:GetKnownCooldown( spellID)
    CDBTooltipTextRight2:SetText("")
    CDBTooltipTextRight3:SetText("")
    CDBTooltipTextRight4:SetText("")
    CDBTooltip:SetSpellByID(spellID)
    return (
            (
                CDBTooltipTextRight2:GetText()
                and (DF:Deformat(CDBTooltipTextRight2:GetText(), SPELL_RECAST_TIME_MIN)
                or   DF:Deformat(CDBTooltipTextRight2:GetText(), SPELL_RECAST_TIME_SEC)
                or   DF:Deformat(CDBTooltipTextRight2:GetText(), SPELL_RECAST_TIME_HOURS)
                or   DF:Deformat(CDBTooltipTextRight2:GetText(), SPELL_RECAST_TIME_CHARGES_DAYS)
                or   DF:Deformat(CDBTooltipTextRight2:GetText(), SPELL_RECAST_TIME_CHARGES_HOURS)
                or   DF:Deformat(CDBTooltipTextRight2:GetText(), SPELL_RECAST_TIME_CHARGES_MIN)
                or   DF:Deformat(CDBTooltipTextRight2:GetText(), SPELL_RECAST_TIME_CHARGES_SEC))
            ) or (
                CDBTooltipTextRight3:GetText()
                and (DF:Deformat(CDBTooltipTextRight3:GetText(), SPELL_RECAST_TIME_MIN)
                or   DF:Deformat(CDBTooltipTextRight3:GetText(), SPELL_RECAST_TIME_SEC)
                or   DF:Deformat(CDBTooltipTextRight3:GetText(), SPELL_RECAST_TIME_HOURS)
                or   DF:Deformat(CDBTooltipTextRight3:GetText(), SPELL_RECAST_TIME_CHARGES_DAYS)
                or   DF:Deformat(CDBTooltipTextRight3:GetText(), SPELL_RECAST_TIME_CHARGES_HOURS)
                or   DF:Deformat(CDBTooltipTextRight3:GetText(), SPELL_RECAST_TIME_CHARGES_MIN)
                or   DF:Deformat(CDBTooltipTextRight3:GetText(), SPELL_RECAST_TIME_CHARGES_SEC))
            ) or (
                CDBTooltipTextRight4:GetText()
                and (DF:Deformat(CDBTooltipTextRight4:GetText(), SPELL_RECAST_TIME_MIN)
                or   DF:Deformat(CDBTooltipTextRight4:GetText(), SPELL_RECAST_TIME_SEC)
                or   DF:Deformat(CDBTooltipTextRight4:GetText(), SPELL_RECAST_TIME_HOURS)
                or   DF:Deformat(CDBTooltipTextRight4:GetText(), SPELL_RECAST_TIME_CHARGES_DAYS)
                or   DF:Deformat(CDBTooltipTextRight4:GetText(), SPELL_RECAST_TIME_CHARGES_HOURS)
                or   DF:Deformat(CDBTooltipTextRight4:GetText(), SPELL_RECAST_TIME_CHARGES_MIN)
                or   DF:Deformat(CDBTooltipTextRight4:GetText(), SPELL_RECAST_TIME_CHARGES_SEC))
            )
           )
end

function spells:GetSpellInfo(index)
    local spellType, spellBookId = GetSpellBookItemInfo(index, BOOKTYPE_SPELL)
    if spellType == "FLYOUT" then
    	return nil, spellBookId, spellType
    end
    
    local spellName = GetSpellBookItemName(index, BOOKTYPE_SPELL)
    local spellLink = GetSpellLink(index, BOOKTYPE_SPELL)
    local spellID   = select(3, string_find(spellLink, "spell:(%d+)"))
    
    local sets = self.db.profile.cooldownSets
    for name, data in pairs(sets) do
        if (data.type == "Spell") and (data.ids[spellID] == true) then
            return name, spellID, data.icon
        end
    end
    return spellName, spellID, GetSpellTexture(index, BOOKTYPE_SPELL)
end
